# Generated by Django 3.2.15 on 2023-01-18 22:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0100_courseflowuser_notifications"),
    ]

    operations = [
        migrations.AddField(
            model_name="courseflowuser",
            name="notifications_active",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="courseflowuser",
            name="notifications",
            field=models.BooleanField(default=False),
        ),
    ]
