# Generated by Django 2.2.25 on 2022-03-22 21:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0082_auto_20220321_1804"),
    ]

    operations = [
        migrations.RenameField(
            model_name="objectset",
            old_name="translation",
            new_name="title",
        ),
    ]
