
#site{
    background:#EEEEEE;
}

p{
    color:black;
}

button, input[type="submit"]{
    border:2px outset #04ba74;
    background:white;
    padding:5px;
    border-radius:5px;
}

:focus{
    outline:none;
}


.workflow-create-wrapper{
    font-family:sans-serif;
    width:100%;
    height:100%;
    background:white;
    padding-top:50px;
    padding-left:40px;
    box-sizing: border-box;
}

#create{
    height: 100%;
}

.workflow-create-wrapper p label, .workflow-create-wrapper p input, .workflow-create-wrapper p textarea{
    vertical-align:top;
    display:block;
}

.workflow-create-wrapper p input, .workflow-create-wrapper p textarea{
    width:400px;
    resize:none;
    margin-top:4px;
    margin-bottom:10px;
}

.workflow-wrapper{
    white-space:nowrap;
    text-align:left;
    width: fit-content;
    background: white;
    padding: 50px;
    padding-top: 35px;
    min-width: 100%;
    min-height: 100%;
    box-sizing: border-box;
}

.workflow-wrapper.printing{
    margin-top:0px;
    background:white;
}

.project-title-in-workflow{
    font-size: 12px;
    text-decoration: underline;
    color: #444444;    
}

#content-container{
    font-family:'Open Sans';
}

.workflow-container{
    font-family:'Open Sans';
    position:relative;
    background:white;
    overflow:visible;
    margin:10px;
    padding:0px;
    user-select:none;
}

.printing .workflow-container{
    margin-right:10px;
}

.workflow-header{
    margin:20px;
    margin-top:50px;
    border-radius:10px;
}

.workflow-header > div{
    margin-left:20px;
}

.parent-workflow-indicators{
    background:white;
    border-radius: 10px;
    padding-right:10px;
    padding-left:10px;
    margin-bottom:8px;
}

.emptytext{
    white-space:normal;
    max-width:800px;
}

.workflow-view-select{
    display:flex;
    gap: 5px;
    flex-wrap:wrap;
    margin:0px 0px 10px 0px;
}

.workflow-view-select > *{
    max-width:200px;
    overflow:hidden;
    padding:8px; 
    font-size:16px;
    border-radius:20px;
    background:white;
    border:1px solid #E9EBF2;
}

.title-text ul, .title-text ol{
    padding-left: 1.5em;
}

.ql-editor ul li, .ql-editor ol li{
    padding-left: 0 !important;
}

.workflow-view-select .title-text{
    white-space: nowrap;
}

.workflow-view-select .active{
    background:#04ba74;
    color:white;
    border:1px solid #04ba74;
}

.workflow-view-select .disabled{
    background:#E9EBF2;
    color:darkgrey;
    pointer-events: none;    
}

.other-views{
    position:relative;
    overflow:visible;
}

.views-dropdown{
    display:none;
    z-index:5;
    background:white;
    position:absolute;
    padding:10px;
    border-radius:5px;
    box-shadow: 0px 2px 2px 0px rgba(0,0,0,0.3);
    margin-top:5px;
    margin-left:-5px;
}

.views-dropdown a{
    display:block;
    margin:2px;
}

.views-dropdown.toggled{
    display:block;
}

.workflow-details{
    /*padding:50px;*/
    width: fit-content;
}
/*
.workflow-details .workflow-for-menu{
    color:black;
    margin-left:10px;
    border-color:black;
    margin-bottom:30px;
}
*/
.workflow-wrapper *:is(.node,.week,.column,.workflow-for-menu,.outcome):not(.selected):not(.valid-drop):hover{
    border-color:gray;
}


.workflow-details .workflow-for-menu.selected{
    background:white;
    color:black;
}

.week{
    background:#f5f5f5;
    min-height:100px;
    padding:10px;
    padding-bottom:24px;
    margin-top:5px;
    box-sizing:border-box;
    border:2px solid white;
    border-radius:5px;
    position:relative;
}

.week > .title-text{
    font-weight:600;
    font-size:14px;
    white-space: normal;
    overflow: hidden;
}

.week-workflow.dragging{
    height:50px;
    overflow:hidden;
    border-width:2px;
    border-radius:5px;
}

.week-workflow.dragging .week{
    border:none;
}

.week-workflow.dragging .node-block{
    display:none;
}


.week-block{
    width:100%;
    padding-left:10px;
    padding-right:10px;
    box-sizing:border-box;
}


.week-block > div:nth-of-type(odd) > .week{
    background: #e0e0e0e0;
}

.week.strategy{
    border:black double;
    margin-left:-1px;
}

.strategy-tab > div{
    display:inline-block;
    vertical-align:top;
    box-sizing:border-box;
}

.strategy-tab  .strategy-tab-square{
    background:#660385;
    padding:4px;
    height:48px;
}

.strategy-tab-triangle{
    width:0;
    height:0px;
    border-top:24px solid transparent;
    border-bottom:24px solid transparent;
    border-right:16px solid #660385;
}

.strategy-tab-circle{
    background:white;
    border-radius:20px;
    width:40px;
    height:40px;
    text-align:center;
}

.strategy-tab{
    position:absolute;
    right:-48px;
    top:calc(50% - 24px);
}

.strategy-tab img{
    width:24px;
    margin-top:calc(50% - 12px);
}

.column-workflow{
    display:inline-block;
    vertical-align:top;
}

.column{
    position:relative;
    padding:5px;
    width:160px;
    white-space: normal;
    text-align:center;
    vertical-align:top;
    box-sizing:border-box;
    border:2px solid white;
    border-radius:5px;
}

.column-line div{
    display:inline-block;
    vertical-align: middle;
    width:calc(100% - 32px);
    padding:2px;
    box-sizing:border-box;
    white-space: normal;
}

.column-line img{
    display:inline-block;
    width:32px;
    vertical-align: middle;
    padding:2px;
    box-sizing:border-box;
}

.column-line .material-symbols-rounded{
    font-size: 32px;
    font-weight: 300;
    vertical-align: middle;
    display: inline-block;
    padding: 2px;
    box-sizing: border-box;
}

.column-line{
    min-height:32px;
    white-space: nowrap;
}

.column-row{
    padding-left:40px;
    padding-right:40px;
    background:white;
    position:sticky;
    top:0px;
    z-index:2;
    font-size:16px;
    border:2px solid white;
    display: flex;
}

.condensed .column-row{
    gap: 60px;
    padding-left: 50px;
    padding-right: 50px;
}

.node-week:first-child {
    padding-top: 5px;
}

.node-week{
    width:100%;
    display:block;
    position:relative;
    min-height:50px;
    padding-top:30px;
}

.node-week.empty{
    min-height:0px;
    padding-top:0px;
}

.node-week.dragging.ui-sortable-helper .node{
    position:static;
}

.node-block{
    min-height:100px;
}

.node-block.term{
    /*width:160px;*/
    width: 220px;
    display:inline-block;
    vertical-align: top;
}

.term .node{
    left:10px !important;
    width:180px;
}


.node-ghost{
    width:160px;
    min-height:50px;
    border:1px dashed black;
    position:absolute;
    cursor:move;
}

.nodeweek-ghost{
    width:160px;
    min-height:50px;
    border:1px dashed black;
    position:absolute;
    cursor:move;
}

.columnworkflow-ghost{
    width:160px;
    min-height:50px;
    border:1px dashed black;
    position:absolute;
    z-index:3;
    cursor:move;
}

.weekworkflow-ghost{
    min-height:50px;
    border:1px dashed black;
    position:absolute;
    cursor:move;
}

.outcomeoutcome-ghost{
    min-width:300px;
    min-height:10px;
    border:1px dashed black;
    position:absolute;
    cursor:move;
}

.outcomeworkflow-ghost{
    min-width:300px;
    min-height:10px;
    border:1px dashed black;
    position:absolute;
    cursor:move;
}

.node{
    width:200px;
    border:solid 1px black;
    position:relative;
    white-space:normal;
    border-radius:5px;
}

.node-top-row{
    padding:5px;
    box-sizing:border-box;
    background:white;
    margin-top:5px;
    width:100%;
    display: flex;
    align-items: center;
    gap: 5px;
}

.node-icon img{
    width:24px;
    vertical-align:middle;
}


.node.program .node-icon{
    width:0;
}

.node-title{
    overflow-x:hidden;
    text-align: center;
    min-height:20px;
    font-weight:600;
    box-sizing:border-box;
    font-size:12px;
    vertical-align:middle;
    flex-grow: 1;
}

.node.program .node-title{
    width:100%;
    text-align: left;
}

.node .assignment-timing{
    background: white;
}

.assignment-timing{
    padding: 5px;
    box-sizing: border-box;
    width: 100%;
    display: flex;
    gap: 5px;
}

.assignment-timing .flatpickr-input+input{
    font-size: 12px;
}

.assignment-in-node .assignment-timing{
    flex-wrap: wrap;
}

.assignment-in-node .workflow-title{
    font-size: 12px;
}

.flatpickr-input+input:disabled{
    background: none;
    border: none;
    color: #444444;
}

.assignment.node{
    width: 600px
}

.linked-workflow.link-noaccess{
    color: "#444444";
}

.linked-workflow{
    background:white;
    width:100%;
    font-size:12px;
    box-sizing:border-box;
}

.linked-workflow.hover-shade{
    color:#1976bc;
}

.linked-workflow > *{
    display:inline-block;
    vertical-align:middle;
}

.linked-workflow > img{
    padding-right:5px;
    padding-left:5px;
    width:12px;
}

.node-details{
    width:100%;
    background:white;
    display:none;
    padding:5px;
    font-size:12px;
    box-sizing:border-box;
}

.node-details default-text{
    font-style:italic;
    color:grey;
}

.node.dropped .node-details{
    display:block
}

.node-drop-row{
    width:100%;
    height:20px;
    background:white;
    border-radius:0 0 5px 5px;
    overflow:hidden;
    cursor:pointer;
}


.week > .node-block{
    display:none;
}

.week.dropped > .node-block{
    display:block;
}

.week-drop-row{
    position:absolute;
    bottom:0px;
    left:0px;
    width:100%;
    height:20px;
    border-radius:0 0 5px 5px;
    overflow:hidden;
    cursor:pointer;
}

.node-drop-side{
    display:inline-block;
    vertical-align: middle;
    height:100%;
    width:40%;
    box-sizing:border-box;
}

.node-drop-left{
    font-size:14px;
    font-weight:600;
    padding-left: 3px;
}

.node-drop-right{
    line-height:100%;
    text-align:right;
    white-space:nowrap;
}

.node-drop-time{
    height:100%;
    font-size:12px;
    text-align:right;
    display:inline-block;
    padding-right:3px;
}

.node-drop-middle{
    display:inline-block;
    vertical-align: middle;
    text-align:center;
    width:20%;
    height:100%;
}

.node-drop-row img,.week-drop-row img{
    width:12px;
    vertical-align:middle;
    display:inline-block;
}

.outcome{
    position:relative;
    margin-left:20px;
    border:2px solid white;
    padding:2px;
    min-width:300px;
    max-width:800px;
}

li.outcome-outcome::before{
    content:"";
    display:block;
    width:10px;
    height:0;
    border-top:2px solid #04ba74;
    margin-top:-2px;
    position:absolute;
    top:15px;
    left:-12px;
}

li.outcome-outcome{
    position:relative;
    margin-left:12px;
    list-style-type:none;
}

li.outcome-outcome > .outcome{
    margin-left:0px;
}

li.outcome-outcome::marker{
    
}

ol.children-block{
    padding-left:0px;
}

.outcome ol ol{
    list-style-type:lower-roman;
}

.outcome.dropped > .outcome-create-child{
    display:block;
}

.outcome.dropped > .children-block:empty + .outcome-create-child{
    display:none;
}

.outcome-create-child{
    position:relative;
    font-size:12px;
    color:#04ba74;
    margin-left:40px;
    display:none;
    cursor:pointer;
}

.outcome-create-child::before{
    content:"";
    display:block;
    width:10px;
    height:6px;
    border-bottom:2px solid #04ba74;
    border-left:2px solid #04ba74;
    position:absolute;
    top:0px;
    left:-20px;
}

.outcome-title{
    padding:2px;
    white-space:normal;
}

.children-block{
    border-left:2px solid #04ba74;
    margin-left:20px;
    margin-top:0px;
    margin-bottom:0px;
    display:none;
    position:relative;
}


.outcome.dropped > .children-block{
    display:block;
}

.outcome-drop{
    margin-left:14px;
    line-height:12px;
    cursor:pointer;
}

.outcome-drop-img{
    display:inline-block;
}

.outcome-drop-text{
    font-size:10px;
    color:#04ba74;
    display:inline-block;
    margin-left:10px;
    vertical-align:top;
}

.outcome-drop-img img{
    width:12px;
    filter: invert(35%) sepia(85%) saturate(855%) hue-rotate(173deg) brightness(91%) contrast(90%);
}

.outcome-edit{
    margin-top:5px;
    display:inline-block;
    vertical-align: top;
    max-width:1000px;
}

.outcome-edit .outcome{
    
}

.right-panel-inner .parent-outcome-node{
    position:relative;
}

.right-panel-inner .parent-outcome-node > *:first-child{
    width:16px;
    left:8px;
    top:6px;
    position:absolute;
}

.right-panel-inner .parent-outcome-node > .outcome{
    margin-left:4px;
    padding-left:24px;
}

.restore-bar-item{
    border:2px solid #04ba74;
    border-radius:5px;
    padding:5px;
}


.outcome-workflow{
    margin-bottom:2px;
}

.outcome-bar-outcome-block .outcome{
    margin-left:0px;
    min-width:0px;
}

.outcome-bar-outcome-block .outcome-outcome{
    margin-bottom:2px;
}

.outcome-bar-outcome-block .children-block{
    margin-left:5px;
    border:0px;
}

.outcome-bar-outcome-block > div > .outcome, .outcome-bar-outcome-block .parent-outcome-node > .outcome{
    border: 1px solid #04ba74;
    border-radius: 5px;
    margin-bottom: 2px;
}

.outcome-bar-outcome-block .outcome-drop{
    margin-left:0px;
}
.outcome-bar-outcome-block .outcome-title, .outcome-node-container .outcome-title{
    font-size:10px;
}

.outcome-bar-outcome-block .outcome-title{
    cursor:pointer;
}

.outcome-ghost{
    width:160px;
    min-height:20px;
    border:1px dashed black;
    position:absolute;
}

.side-actions{
    position:absolute;
    top:0px;
    right:-22px;
}
/*
.competency-matrix .nodewrapper .side-actions{
    top: -202px;
    left: 0px;
    right: auto;
}*/

.outcome-table .nodewrapper .side-actions{
    top: -160px;
    right: -70px;
}

.side-actions .comment-indicator{
    position: unset;
    background: unset;
}

.outcome-node-container{
    display:none;
    position:absolute;
    top:0px;
    left:0px;
    background:white;
    border:2px solid;
    border-radius:10px;
    z-index:1;
    background:white;
    width:200px;
}

.outcome-node-indicator-number{
    display:inline-block;
    min-width:16px;
    height:16px;
    background:white;
    border: 2px solid;
    border-radius:10px;
    text-align:center;
}

.outcome-node-indicator:hover .outcome-node-container{
    display:inline-block;
}

.outcome-node-container .outcome{
    min-width:initial;
    max-width:initial;
    margin:0px;
    border:0px;
}

.outcome-node-container .children-block{
    margin-left:5px;
    border:0px;
}

.outcome-node-container .outcome-drop{
    margin-left:0px;
}

.outcome-node-container .outcome-node{
    position:relative;
}

.outcome-node img{
    width:16px;
}

.outcome-node >.outcome>.outcome-title{
    margin-left:0px;
}

.outcome-node >.outcome{
    margin-left:0px;
}

.outcome-node{
    display:flex;
    align-items:center;
}

.outcome-node .outcome-degree{
    width:16px;
    text-align:center;
}

.outcome-degree{
    font-family:serif;
}

.outcome-node-container .outcome-title{
    margin-left:16px;
}

.outcome-node-container .action-button{
    width:12px;
    height:12px;
    margin-left:4px;
}

.outcome-node-container .mouseover-actions{
    right:auto;
    left:2px;
    top:2px;
}

.outcome-node-container .outcome-outcome{
    position:relative;
    left:4px;
}

.outcome-hover, .outcome-toggle{
    outline:2px solid #ffff80;
}

.outcome-toggle-checkbox{
    position:absolute;
    right:-2px;
    top:-2px;
    display:none;
}

.outcome-hover>.outcome-toggle-checkbox{
    display:block;
}

.outcome-bar-outcome-block .outcome-outcome{
    background:white;
}

.outcome-table{
    margin-right:80px;
}

.outcome-table .outcome{
    max-width:none;
    padding:0;
    border:0px;
    font-size: 12px;
    margin-left:0px;
}

.outcome-table .children-block{
    margin-left:0px;
    border-left:0px;
}

.outcome-table .outcome-title{
    padding-bottom:0px;
    padding-top:1px;
}

.outcome-table .outcome-drop{
    margin-top:-2px;
    margin-left:-2px;
}

.outcome-table .node-row .outcome-head{
    background:white;
    position:sticky;
    left:0px;
    height:100%;
    z-index:1;
}

.outcome-table .table-cell, .competency-matrix .table-cell{
    width:30px;
    min-height:24px;
    border-left:1px solid black;
    vertical-align:top;
    text-align:center;
    position:relative;
    box-shadow:0px 1px 0 0 #dddddd;
}

.outcome-table .table-cell{
    display: inline-block;
    height:100%;
}


.outcome-table .blank-cell{
    box-shadow:none;
}

.outcome-table .horizontal-table-node{
    position:absolute;
    font-weight:600;
    font-size:14px;
    border-bottom:1px solid black;
    padding:4px;
    padding-top:5px;
    top:4px;
    width:32px;
    box-sizing:border-box;
    overflow:hidden;
    z-index:1;
}

.outcome-table .horizontal-table-header{
    display:inline-block;
    position:relative;
}

.outcome-table .total-cell, .outcome-table .total-header, .competency-matrix .total-cell, .competency-matrix .total-header{
    background:#EAEAEA;
}

.outcome-table .table-cell.disabled, .outcome-table .table-cell.disabled .child-outcome{
    background:#BABABA;
}
.outcome-table .table-cell.not-first-child-outcome{
    border-left:2px solid transparent;
}
.outcome-table .table-cell.not-first-child-outcome .child-outcome{
    border-top:2px solid transparent;
}

.outcome-table .grand-total-cell{
    border-right:1px solid black;
}

.outcome-table.node-rows .outcome-head.user-head{
    width: 200px;
    border-right: none;
}

.user-table .table-cell input{
    display: block;
}

.user-table .table-cell .workflow-title{
    font-size: 12px;
}

.user-table{
    white-space: nowrap;
}

/*
.competency-matrix .outcome{
    font-size:14px;
    transform-origin:top left;
    transform: rotate(-90deg);
    position:absolute;
    margin:0px;
    border-bottom:2px solid black;
    text-align:left;
    height:30px;
    box-sizing:border-box;
    left:2px;
    min-width:revert;
    width:180px;
}

.competency-matrix .outcome .title-text{
    overflow:hidden;
    white-space:normal;
    width:100%;
    height:100%;
}

.competency-matrix .outcome-drop{
    margin:0px;
    left:-20px;
    top:8px;
    position:absolute;
}

.competency-matrix .outcome-block{
    height:100%;
    display:inline-block;
}

.competency-matrix .outcome-block.dropped > .outcome-block-children{
    display:inline-block;
}

.competency-matrix .outcome-block-children{
    height:100%;
    display:none;
}

.competency-matrix .outcome-row.node-row{
    padding-top:200px;
}

.competency-matrix .node{
    width:100%;
    height:26px;
    display:inline-block;
    vertical-align:top;
}

.competency-matrix .week .node-row:first-child .outcome-header{
    background:#E0E0E0;
}

.competency-matrix .week{
    background:none;
    min-height:revert;
    padding:0;
    margin:0;
    border:0px;
}


.competency-matrix .outcome-head{
    display:inline-block;
    width:160px;
}*/


.competency-matrix .outcome-row.node-row{
    position:sticky;
    left:0px;
    z-index:2;
    background:white;
}

.competency-matrix .outcome-row.node-row .table-cell{
    width: 202px;
    border-left: 0px;
}

.competency-matrix .table-body{
    display: inline-block;
    vertical-align: top;
}

.competency-matrix .node-top-row{
    padding: 2px;
    border-radius: 0 0 5px 5px;
}

.competency-matrix .outcome-row{
    display: inline-block;
    position: relative;
    vertical-align: top;
}

.competency-matrix .week .node-row:first-child{
    background:#E0E0E0;
}

.competency-matrix .table-cell{
    height:30px;
}

.competency-matrix .outcome-wrapper{
    width: 30px;
    position: sticky;
    top: 180px;
    z-index: 3;
}

.competency-matrix .outcome-head{
    font-size:14px;
    transform-origin:top left;
    transform: rotate(-90deg);
    position:absolute;
    margin:0px;
    border-top:1px solid black;
    text-align:left;
    height:30px;
    box-sizing:border-box;
    left:0px;
    min-width:revert;
    width:180px;    
    background: white;
}

.competency-matrix .matrix-time-block{
    display: inline-block;
}

.competency-matrix .matrix-time-block .outcome-head{
    left: -1px;
}

.matrix-time-block > .matrix-time-row:first-child {
    position: sticky;
    top: 180px;
    z-index: 3;
}

.competency-matrix .matrix-time-block .table-cell{
    display: inline-block;
}

.competency-matrix .node-row .outcome-head{
    border: 0px;
    height: 203px;
    left: 0px;
}

.competency-matrix .outcome-title{
    padding-top: 0px;
    max-height: 20px;
    overflow-y: hidden;
}

.competency-matrix .outcome-drop{
    position: relative;
    top: -5px;
}

.competency-matrix .outcome-row+.outcome-row:not(.depth-0) .table-cell{
    border-left: 0px;
}

.competency-matrix .outcome-row+.outcome-row:not(.depth-0) .outcome-head{
    border-top: 0px;
}

.competency-matrix{
    padding: 40px;
    padding-top: 220px;

}

.workflow-grid{
    display:flex;
}

.workflow-grid .week{
    margin-right: 20px;
}

.workflow-grid .node-top-row{
    border-radius:0 0 5px 5px;
}

.grid-ponderation{
    font-size:14px;
    font-weight:600;
    margin:5px;
}

.outcome-row:not(.node-row):hover, .outcome-row:not(.node-row):hover .total-cell{
    background:#E0E0E0;
}

.outcome-row:not(.node-row):hover .outcome-head{
    background: #E0E0E0;
}

.table-cell .outcome-degree{
    display:inline-block;
    font-family:serif;
    font-weight:600;
    white-space:nowrap;
    margin-top:6px;
    opacity:0.7;
}
.outcome-introduced{
    color:#167016;
}
.outcome-introduced.self-completed{
    color:white;
    background:#167016;
    opacity:1;
    min-width:10px;
}
.outcome-developed{
    color:#2ca02c;
}
.outcome-developed.self-completed{
    color:white;
    background:#2ca02c;
    opacity:1;
    min-width:10px;
}
.outcome-advanced{
    color:#11ca11;
}
.outcome-advanced.self-completed{
    color:white;
    background:#11ca11;
    opacity:1;
    min-width:10px;
}

.outcome-table .table-group{
    display:inline-block;
    height:100%;
}

.outcome-cells{
    display:inline-block;
    vertical-align: top;
}

.outcome-table.node-rows .outcome-head{
    height: 100%;
}

.outcome-table .outcome-wrapper{
    display:inline-block;
    width:280px;
    position:sticky;
    left:0px;
    background:white;
    z-index:1;
    box-sizing:border-box;
    border-right:2px solid #E9EBF2;
}

.outcome-table .outcome-head.empty{
    border-right-color:white;
    position:relative;
    width: 280px;
}

.outcome-table.node-rows .outcome .outcome .outcome-head{
    border-left:2px solid #04ba74;
}

.outcome-row{
    display:flex;
}


.outcome-table .outcome-row.node-row{
    padding-top:140px;
    position:sticky;
    top:0px;
    z-index:2;
    background:white;
}


.outcome-table .outcome-row.outcome-category{
    border-bottom:2px solid #E9EBF2;
    border-top:2px solid #E9EBF2;
}

.outcome-row.outcome-category h4{
    margin-left:5px;
    margin-top:0px;
    margin-bottom:0px;
}

.outcome-table .table-cell.nodewrapper{
    position:relative;
}

.table-cell > img{
    padding:5px;
    width:16px;
    opacity:0.7;
}

.table-cell > img.self-completed{
    opacity:1;
}

.table-cell input, .table-cell button, .table-cell select{
    position:absolute;
    top:3px;
    right:4px;
    display:none;
}

.table-cell select:focus{
    display: inline-block;
}

.table-cell select{
    width: 30px;
    right: 0px;
}

.table-cell button{
    background:white;
    box-shadow:none;
    border:1px solid black;
    border-radius:3px;
}

.table-cell:hover input, .table-cell:hover button, .table-cell:hover select{
    display:inline-block;
}

.outcome-table.node-rows .node, .outcome-table.node-rows .child-outcome{
    width:160px;
    position:absolute;
    left:-2px;
    height:26px;
    transform-origin:top left;
    transform:rotate(-60deg);
    
}

.outcome-table .child-outcome{
    border-top:1px solid black;
    text-align:left;
    padding-left:10px;
    white-space:normal;
    overflow:hidden;
    font-size:11px;
    font-weight:600;
}


.outcome-table .node-title{
    width:100%;
    min-height:0px;
    max-height:18px;
}

.outcome-table .node-top-row{
    padding:1px;
    border-radius:0px 0px 5px 5px;
    height: calc(100% - 5px);
}

.outcome-table .table-cell .total-header, .outcome-table .table-cell .node-category-header{
    position:absolute;
    width:160px;
    height:26px;
    transform-origin:top left;
    transform:rotate(-60deg);
    border-bottom:0px;
    font-weight:600;
    text-align:left;
    font-size:10px;
    white-space:normal;
    background: inherit;
    border-top: 1px solid black;
    left: -1px;
}

.outcome-table .total-header::before, .outcome-table .node-category-header::before{
    content:"";
    width:16px;
    height:5px;
    display:inline-block;
}

.outcome-table .total-header::after{
    content:"";
    position:absolute;
    margin-left:14px;
    border-bottom:1px solid black;
    bottom:-2px;
    width:calc(100% - 14px);
    left:0px;
}

.outcome-table .table-cell .node-category-header{
    font-size:12px;
    overflow-y: auto;
    overflow-x: hidden;
}

.outcome-table .table-group:first-child .blank-cell{
    border-left:0px;
}

.viewbar .outcomeviewtext{
    display:block;
    white-space: nowrap;
}

.switch{
    position: relative;
    display:inline-block;
    width:60px;
    height:16px;
}

.slider{
    position:absolute;
    cursor:pointer;
    top:0;
    left:0;
    right:0;
    bottom:0;
    background-color:grey;
    transition:0.4s;
    border-radius:34px;
}

.slider::before{
    border-radius:50%;
    position:absolute;
    content:"";
    height:12px;
    width:12px;
    left:4px;
    bottom:2px;
    background-color:white;
    -webkit-transition:.4s;
    transition:.4s;
}


input:checked + .slider{
    background-color:#04ba74;
}

input:checked +.slider::before{
    -webkit-transform:translateX(40px);
    -ms-transform:translateX(40px);
    transform:translateX(40px);
}

.mouseover-actions{
    visibility:hidden;
    position:absolute;
    top:-1px;
    right:-1px;
    border-radius:5px;
    border:0px solid black;
    padding:2px;
    line-height:0px;
    background:white;
}

/*This is used to get around the issue that we can't make the week itself relatively positioned (messes up jquery sortable*/
.mouseover-container-bypass{
    position:relative;
    padding:0;
    margin:0;
    top:0px;
    width:100%;
    height:0px;
    overflow:visible;
}

.action-button{
    width:20px;
    height:20px;
    display:inline-block;
    cursor:pointer;
}

.action-button img{
    width:100%;
    height:100%;
}


div:hover > .mouseover-actions,div:hover > .mouseover-container-bypass > .mouseover-actions{
    visibility:visible;
}


.dragging .mouseover-actions{
    visibility:hidden !important;
}

.comment-box{
    text-align:left;
    position:absolute;
    width:300px;
    max-height:400px;
    border:1px solid #04ba74;
    background:white;
    z-index:7;
    overflow: overlay;
    padding: 5px;
    font-size: 12px;
    box-sizing: border-box;
    top:0px;
    right:-305px;
    border-radius:5px;
    box-shadow: 0px 1px 2px 1px rgba(0,0,0,0.3);
}

.comment-top-row{
    display: flex;
    flex-direction: row-reverse;
    gap: 16px;
}

.comment-top-row img{
    width: 20px;
}

.comment-block{
    width: 100%;
    box-sizing: border-box;
}

.comment .mouseover-actions{
    top: 1px;
    right: 1px;
}

.comment .action-button{
    width: 12px;
    height: 12px;
}

.comment-box .comment-input-line{
    width:100%;
    box-sizing: border-box;
    margin:0px;
    margin-top:4px;
}

.add-comment-button{
    width: 20px;
    margin: 10px;
}

.add-comment-button.hidden{
    display: none;
}

.comment-input-line{
    display: flex;
    align-items: center;
}

.comment-box button.small{
    padding:4px;
    font-size:10px;
}

.comment-box .comment-input{
    min-height: 60px;
    max-height:100px;
    white-space:normal;
    background: #EEEEEE;
    flex-grow: 99;
    border: none;
    border-radius: 5px;
    resize: none;
}

.comment-text{
    white-space:pre-line;
}

.comment{
    position:relative;
    margin-top:5px;
    padding:2px;
    padding-bottom:5px;
    border:1px solid white;
    box-sizing: border-box;
    width: 100%;
    overflow-x: hidden;
    color: #222222;
    user-select: text;
}

.comment:hover{
    border:1px solid #E9EBF2;
    
}

.comment-by{
    display: flex;
    justify-content: space-between;
}
.comment-on{
    font-size:10px;
    color: grey;
}
.comment-user{
    font-size:14px;
    font-weight: 600;

}

.comment-indicator{
    position:absolute;
    right:0px;
    top:0px;
    border-radius:5px;
    background:white;
    height:20px;
}

.week > .comment-indicator{
    margin-right:10px;
    margin-top:10px;
}

.comment-indicator img{
    width:20px;
    filter: brightness(0) saturate(100%) invert(38%) sepia(98%) saturate(461%) hue-rotate(164deg) brightness(87%) contrast(100%);
}

.outcome-node-container .comment-indicator img{
    width: 12px;
}

.nomenclature-row{
    position:relative;
    width:fit-content;
    padding:8px;
    padding-left:0px;
    display: flex;
    gap: 20px;
    align-items: center;
}

.nomenclature-row div:first-child{
    width: 150px;
}


.nomenclature-row input, .nomenclature-row select {
    padding: 10px;
    border-radius: 3px;
    border: 1px solid black;
}

.workflow-canvas{
    position:absolute;
    left:0px;
    top:0px;
    pointer-events:none;
}

.nodelinkwrapper{
    width:100px;
    height:100px;
    overflow:hidden;
    pointer-events: none;
    display:flex;
    justify-content:center;
    align-items:center;
}

.nodelinktext {
    text-anchor: middle;
    dominant-baseline: middle;
    max-width: 100px;
    max-height: 100px;
    white-space: normal;
    background: white;
    padding: 2px;
    border: 1px solid black;
    font-size: 10px;
    text-align: center;
    width:fit-content;
    pointer-events:all;
}


.workflow-canvas circle, .workflow-canvas path.nodelink{
    pointer-events:auto;
}

.workflow-canvas.creating-node-link circle[data-port-type='target']{
    visibility:visible;
}

.workflow-canvas circle{
    visibility:hidden;
}

circle.mouseover{
    visibility:visible;
}

.workflow-canvas.dragging-weekworkflow{
    opacity:0;
}
/*
.node input{
    width:100px;
}*/

.side-bar{
    z-index:4;
    height: 100%;
    max-width:260px;
    position: relative;
}


.right-panel-container{
    float:right;
    box-shadow: -1px 0px 1px -1px;
    background:white;
    text-align:left;
    font-family:sans-serif;
    height:100%;
    width:260px;
    position:relative;
}

.side-bar hr{
    margin: 10px;
    margin-left: 10%;
    margin-right: 10%;
}

.side-bar .ui-tabs-tab.hidden{
    display: none;
}

.side-bar.collapsed .window-close-button{
    display:none;
}

.side-bar .ui-tabs-nav{
    margin:5px -2px 5px 5px; 
    padding:0px;
    left:-40px;
    top: 36px;
    position:absolute;
}

.side-bar .ui-tabs-tab{
    text-align:center;
    display:block;
    background:white;
    border-radius:3px;
    margin: 1px;
    margin-bottom: 2px;
    padding: 6px;
    box-sizing: border-box;
    box-shadow: 0px 4px 4px 2px #e6e6e6;
    color: #04ba74;
}

.side-bar .ui-tabs-tab.ui-state-disabled{
    background-color:#E9EBF2;
    cursor:default;
    color: #B5B5B5;
}


.side-bar .ui-tabs-tab a span{
    font-size: 20px;
    margin-top: 2px;
    margin-left: 1px;
}

.side-bar .ui-tabs-tab a{
    margin-right:4px;
    display:block;
    text-decoration:none;
    box-sizing:border-box;
    height:24px;
    color: inherit;
}


.side-bar .ui-tabs-tab.ui-tabs-active{
    color: white;
    background: #04ba74;
}



.side-bar select, .side-bar input[type=text], .side-bar textarea, .side-bar input[type=number]{
    width:100%;
    box-sizing:border-box;
    padding:5px;
    border:1px solid lightgrey;
    margin-top:2px;
    margin-bottom:2px;
}

.side-bar input[type=number]{
    -moz-appearance:textfield;
}

.side-bar input::-webkit-outer-spin-button,
.side-bar input::-webkit-inner-spin-button {
    -webkit-appearance: none;
    margin: 0;
}

.side-bar button{
    margin-top:2px;
    margin-bottom:2px;
    box-sizing:100%;
    width:100%
}

.side-bar input[type=text]{
    padding-top:6px;
    padding-bottom:6px;
}


.side-bar .quill-div{
    min-height:140px;
    position:unset;
}

.side-bar #title-editor{
    min-height:60px;
    max-height: 100px;
}

.side-bar .character-length{
    font-size:10px;
    color:grey;
    float: right;
}

.side-bar .character-length.overflow{
    color:red;
}

.side-bar input.half-width, .side-bar select.half-width{
    width:50%;
}

.side-bar .delete-self-button{
    margin:5px;
    text-align:center;
}

.half-width{
    width:50%;
    display:inline-block;
    box-sizing:border-box;
}

.right-panel-container textarea{
    font-family:sans-serif;
    resize:none;
}

.right-panel-inner{
    overflow-y: overlay;
    display:inline-block;
    padding:20px;
    box-sizing:border-box;
    width:100%;
    height:100%;
}

.slidecontainer{
    width:100%;
}

.range-slider{
  appearance: none;
  width: 100%; /* Full-width */
  height: 25px; /* Specified height */
  background: #d3d3d3; /* Grey background */
  outline: none; /* Remove outline */
  opacity: 0.7; /* Set transparency (for mouse-over effects on hover) */
  transition: opacity .2s;
  border-radius:20px;
}

.range-slider:hover {
  opacity: 1; /* Fully shown on mouse-over */
}


.node-bar-column-workflow{
    width:100%;
    display:block;
    margin:2px;
}

.node-bar-column-workflow .ui-sortable-placeholder{
    display:none;
}

.node-bar-column{
    font-size:14px;
    width:100%;
    cursor:move;
    padding:8px;
    border-radius: 5px;
    color: white;
    box-sizing: border-box;
}

.node-bar-column.new-column{
    color: black;
    border: 2px black;
}

.new-column.node-bar-column{
    border-style:dashed;
}

.new-node-drop-over{
    border-bottom:2px solid #04ba74;
}

.outcome-drop-over{
    border:2px solid #04ba74;
}

.new-strategy-drop-over{
    border-bottom:2px solid #04ba74;
}

.valid-drop{
    outline:2px solid #04ba74;
}

.node-bar-week{
    padding: 2px;
    font-weight: 600;
    margin-bottom: 2px;
}

.strategy-bar-strategy{
    width:100%;
    display:block;
    margin:2px;
    font-size:14px;
    padding:8px;
    cursor:move;
    box-sizing:border-box;
    white-space:nowrap;
    background: #660385;
    border-radius: 5px;
    color: white;
}

.strategy-bar-strategy img{
    width:24px;
    display:inline-block;
    vertical-align:middle;
}
.strategy-bar-strategy div{
    display:inline-block;
    vertical-align:middle;
    margin-left:8px;
    white-space:normal;
}

.week-ghost{
    width:300px;
    min-height:50px;
    border:1px dashed black;
    position:absolute;
}

.workflow-wrapper .selected{
    border-color:#04ba74;
    border-style:solid;
}

.workflow-wrapper .nodelink.selected{
    stroke:#04ba74;
    opacity:1;
    stroke-width:5px;
}

.workflow-wrapper .node.selected{
    border-width:2px;
    margin:-1px;
}

.workflow-for-menu.selected{
    border-color:#04ba74;
}

.alignment-block{

}

.alignment-block > .outcome > .mouseover-actions{
    display:none !important;
}

.alignment-block .node{
    width:100%;
    white-space:nowrap;
}

.alignment-block .week .node-block{
    display: block;
}

.alignment-block .node-title{
    width:200px;
    vertical-align: middle;
    display:inline-block;
}

.alignment-block .outcomenode-block{
    display:inline-block;
    vertical-align: middle;
    max-width:600px;
}

.alignment-block .outcomenode-block .mouseover-actions{
    display:none !important;
}

.alignment-block .node-drop-row{
    height:auto;
    padding-left:20px;
    border-top:2px solid #E9EBF2;
    box-sizing:border-box;
}

.alignment-block .parent-outcome-box,.alignment-block .child-outcome-box{
    display:inline-block;
    vertical-align:top;
    white-space:normal;
}

.alignment-block .parent-outcome-box{
    width:30%;
}
.alignment-block .child-outcome-box{
    width:70%;
}

.alignment-block .outcome-alignment{
    white-space:nowrap;
    max-width:800px;
    box-sizing:border-box;
    padding:8px;
    margin:2px;
    border:2px solid white;
}

.alignment-block .outcome-alignment:hover{
    border-color:gray;
}

.alignment-block img{
    width:16px;
}

.alignment-row{
    margin-bottom:4px;
}

.alignment-row > *{
    vertical-align:top;
    margin-right:4px;
}

.alignment-row > *:not(img):not(.outcome-degree){
    min-width:0px;
    width:calc(100% - 50px);
}

.alignment-column .outcome{
    min-width:0px;
    width:calc(100% - 50px);
}

.alignment-column{
    min-width:400px;
}

.alignment-show-all{
    color:#04ba74;
    cursor:pointer;
}

.alignment-block .outcome-block{
    background-color:white;
}

.alignment-block .child-outcome{
    padding-bottom:5px;
    border-top:2px solid #E9EBF2;
}

.alignment-block .child-outcome.child-outcome-header{
    font-weight: 600;
    padding-left: 5px;
}

.alignment-block .child-outcome .half-width{
    vertical-align: top;
}

.alignment-block .child-outcome{
    font-size: 12px;
}

.alignment-block .alignment-added-outcomes{
    font-size: 12px;
}

.alignment-block .node-outcomes-header{
    font-weight: 600;
}

#popup-container{
    position:absolute;
    width:100%;
    height:100%;
    pointer-events:none;
    z-index:10;
    top: 0;
    left: 0;
}


.multi-select{
    display:inline-block;
    width:250px;
}

.multi-select select{
    height:200px;
}

.multi-select >*{
    display:block;
    width:100%;
}

.message-panel{
    width:50%;
    text-align:left;
    box-sizing: border-box;
    display:inline-block;
    background:white;
    height:100%;
    padding:20px;
    overflow: overlay;
    vertical-align:top;
}

.action-bar{
    display:flex;
    gap: 10px;
    right:15px;
    bottom:15px;
    position:absolute;
}

.workflow-legend{
    cursor:move;
    position:fixed;
    background:white;
    z-index:5;
    padding:20px;
    border:2px solid #04ba74;
    left:calc(100% - 460px);
    box-shadow: 0px 2px 2px 0px rgba(0,0,0,0.3);
}

.workflow-legend .legend-section{
    
}

.workflow-legend .legend-line{
    width:100%;
    padding:2px;
}

.workflow-legend .legend-line img{
    display:inline-block;
    vertical-align: middle;
    width:24px
}

.workflow-legend .legend-line div{
    display:inline-block;
    vertical-align: middle;
    margin-left:8px;
    font-weight:600;
    font-size:12px;
}

/*ComparisonView stuff*/

.comparison-view{
    display:flex;
    gap:20px;
    padding:50px;
}

.comparison-view .workflow-description, .comparison-view .workflow-publication{
    display: none;
}

.comparison-view .workflow-title{
    font-size: 12px;
}

.comparison-view .workflow-array{
    display:flex;
    gap:20px;
}

.comparison-view .node-week{
    padding-top:5px;
    max-width:204px;
    white-space:normal;
}

.comparison-view .node-top-row{
    border-radius:0 0 5px 5px;
}

.comparison-view .week{
    min-height:60px;
    box-sizing:content-box;
}

.comparison-view .node-block{
    min-height:30px;
}

.comparison-view .workflow-container{
    padding:0;
    box-shadow:none;
    margin:0;
}

.comparison-view .workflow-header{
    margin:0px;
    position:sticky;
    z-index:1;
    top:0px;
    border:1px solid black;
    display:flex;
}

.comparison-view .workflow-details{
    padding:5px;
    box-sizing: border-box;
    width: 100%;
}

.comparison-view .workflow-wrapper{
    margin:0px;
    width:260px;
    position: relative;
    min-width: auto;
    padding: 0;
}

.comparison-view .workflow-for-menu{
    margin:0px;
    width:100%;
    padding: 5px;
}

.comparison-view .outcome{
    margin-left: 0;
    min-width: unset;
    max-width: 210px;
    font-size: 12px;
}

.workflow-wrapper .window-close-button {
    right: -8px;
    top: -8px;
    background: white;
    height: 16px;
    width: 16px;
    border-radius: 8px;
    z-index: 2;
}


/*
.workflow-legend *:is(h4, h5), .right-panel-container *:is(h3, h4, h5){
    color:#04ba74;
}
*/
.selectable{
    user-select: unset;
}

.load-screen{
    z-index:99;
    position:absolute;
    left:0;
    top:0;
    width:100%;
    height:100%;
    background:#80808060;
}
.load-screen::after{
    position:absolute;
    left:50%;
    top:50%;
    width:50px;
    height:50px;
    margin:-25px 0 0 -25px;
    content:"";
    border-radius:50%;
    border:16px solid #80808010;
    border-top:16px solid #04ba74;
    border-bottom:16px solid #04ba74;
    animation: spin 2s linear infinite;
    -webkit-animation: spin 2s linear infinite;
}

@-webkit-keyframes spin {
  0% { -webkit-transform: rotate(0deg); }
  100% { -webkit-transform: rotate(360deg); }
}

@keyframes spin {
  0% { transform: rotate(0deg); }
  100% { transform: rotate(360deg); }
}

*.waiting, *.waiting *{
    cursor:wait !important;
}


