# Generated by Django 2.2.20 on 2021-06-21 21:54

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0058_auto_20210506_2247"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="project",
            name="outcomes",
        ),
        migrations.AlterField(
            model_name="outcomehorizontallink",
            name="outcome",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="outcome_horizontal_links",
                to="course_flow.Outcome",
            ),
        ),
        migrations.AlterField(
            model_name="outcomehorizontallink",
            name="parent_outcome",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="reverse_outcome_horizontal_links",
                to="course_flow.Outcome",
            ),
        ),
        migrations.DeleteModel(
            name="OutcomeProject",
        ),
    ]
