# Generated by Django 3.2.15 on 2023-02-05 16:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0102_auto_20230205_1612"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="activity",
            name="author",
        ),
        migrations.RemoveField(
            model_name="course",
            name="author",
        ),
        migrations.RemoveField(
            model_name="program",
            name="author",
        ),
    ]
