# Generated by Django 3.2.14 on 2022-09-30 19:41

import uuid

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0089_auto_20220930_1527"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="hash",
            field=models.UUIDField(
                default=uuid.uuid4, editable=False, null=True
            ),
        ),
    ]
