# Generated by Django 2.2.16 on 2021-03-15 19:52

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="course",
            name="discipline",
        ),
        migrations.AddField(
            model_name="project",
            name="discipline",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="course_flow.Discipline",
            ),
        ),
    ]
