# Generated by Django 3.2.15 on 2023-01-18 21:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0099_alter_courseflowuser_user"),
    ]

    operations = [
        migrations.AddField(
            model_name="courseflowuser",
            name="notifications",
            field=models.PositiveIntegerField(
                choices=[(0, "Unknown"), (1, "No"), (2, "Yes")], default=0
            ),
        ),
    ]
