# Generated by Django 2.2.25 on 2022-01-12 22:55

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0077_workflow_condensed"),
    ]

    operations = [
        migrations.AddField(
            model_name="column",
            name="deleted_on",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name="node",
            name="deleted_on",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name="nodelink",
            name="deleted_on",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name="outcome",
            name="deleted_on",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name="project",
            name="deleted_on",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name="week",
            name="deleted_on",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name="workflow",
            name="deleted_on",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]
