# Generated by Django 2.2.20 on 2021-06-21 22:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0059_auto_20210621_2154"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="outcome",
            name="disciplines",
        ),
        migrations.RemoveField(
            model_name="outcome",
            name="published",
        ),
    ]
