{% extends "course_flow/base.html" %}
<!-- -->
{% load static i18n %}
<!-- -->
{% block header %}{% endblock %}
<!-- -->
{% block title %}
  {%if object.title != "" %}
    {{ object.title }}
  {% else %}
    {% trans "Untitled" %}
  {% endif %}
  - CourseFlow
{% endblock %}
<!-- -->
{% block metadescription %} {% endblock %}
<!-- -->
{% block body %}
<!-- -->
{% endblock %}
<!-- -->
{% block foot %} {% endblock %}
<!-- -->
{% csrf_token %}
<!-- -->
{% block read_only_scripts %}

{% endblock %}
<!-- -->
{% block scripts %} {{ block.super }}
<script nonce="{{request.csp_nonce}}">

  const iconpath = "{% static 'course_flow/img/images_svg/' %}";

  const assignment_data= {{assignment_data|safe}};
  const live_project_data= {{live_project_data|safe}};
  const user_role = {{user_role|safe}};
  var live_assignment_renderer;

  const user_id = {{ user.id }};

  window.addEventListener("load",function(){
      live_assignment_renderer = new live_renderers.LiveAssignmentRenderer(assignment_data,live_project_data);
      $(document).ajaxError(renderers.fail_function);
      live_assignment_renderer.render($("#container"));
  });
</script>
{% endblock %}
