{% extends "course_flow/base.html" %}
<!-- -->
{% load static i18n %}
<!-- -->
{% block header %}
{% endblock %}
<!-- -->
{% block title %}{% trans "Explore" %} - CourseFlow{% endblock %}
<!-- -->
{% block metadescription %} {% endblock %}
<!-- -->
{% block body %}
<!-- -->
{% endblock %}
<!-- -->
{% block foot %} {% endblock %}
<!-- -->
{% csrf_token %}
<!-- -->
{% block scripts %} {{ block.super }}

<script nonce="{{request.csp_nonce}}">

  const iconpath = "{% static 'course_flow/img/images_svg/' %}";

  var library_renderer;
  const user_id = {{ user.id }};

  const disciplines_list = {{disciplines|safe}};

  const initial_workflows = {{initial_workflows|safe}};
  const initial_pages = {{initial_pages|safe}};

  window.addEventListener("load",function(){
      explore_renderer = new library_renderers.ExploreRenderer(disciplines_list,initial_workflows,initial_pages);
      $(document).ajaxError(renderers.fail_function);
      explore_renderer.render($("#container"));
      makeActiveSidebar("#panel-explore");
  });
</script>



<!-- <script nonce="{{request.csp_nonce}}">
    
    const iconpath = "{% static 'course_flow/img/images_svg/' %}";
    const explore_path = "{% url 'course_flow:explore' %}";
    const user_id = {{ user.id }};
    
    list_objects = {{object_list|safe}};
    
    disciplines = {{disciplines|safe}};
    pages = {{pages|safe}}
    
    
  jQuery('<div>',{
    class:'load-screen'
  }).appendTo("#container");

    
    const read_only=true;
    
    
    
    window.addEventListener("load",function(){
      renderers.renderExploreMenu(list_objects,disciplines);
      makeActiveSidebar("#panel-explore");
    });
    
    
</script> -->


{% endblock %}
