# Generated by Django 3.2.15 on 2023-02-05 19:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0105_auto_20230205_1753"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="is_strategy",
            field=models.BooleanField(default=False),
        ),
    ]
