# Generated by Django 3.2.15 on 2023-01-18 14:13

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("course_flow", "0097_userassignment_completed_on"),
    ]

    operations = [
        migrations.CreateModel(
            name="CourseFlowUser",
            fields=[
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        primary_key=True,
                        serialize=False,
                        to="auth.user",
                    ),
                ),
                (
                    "first_name",
                    models.CharField(blank=True, max_length=200, null=True),
                ),
                (
                    "last_name",
                    models.CharField(blank=True, max_length=200, null=True),
                ),
            ],
        ),
        migrations.AlterField(
            model_name="program",
            name="author",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="authored_programs",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="author",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="authored_projects",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]
