# Generated by Django 2.2.25 on 2022-02-28 18:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0078_auto_20220112_2255"),
    ]

    operations = [
        migrations.AlterField(
            model_name="column",
            name="title",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="customterm",
            name="term",
            field=models.CharField(max_length=200),
        ),
        migrations.AlterField(
            model_name="customterm",
            name="translation",
            field=models.CharField(max_length=200),
        ),
        migrations.AlterField(
            model_name="customterm",
            name="translation_plural",
            field=models.CharField(max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="node",
            name="title",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="objectpermission",
            name="permission_type",
            field=models.PositiveIntegerField(
                choices=[
                    (0, "None"),
                    (1, "View"),
                    (2, "Edit"),
                    (3, "Comment"),
                ],
                default=0,
            ),
        ),
        migrations.AlterField(
            model_name="outcome",
            name="code",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="project",
            name="title",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="week",
            name="title",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="workflow",
            name="code",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name="workflow",
            name="title",
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
    ]
