# Generated by Django 3.2.15 on 2023-06-01 18:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0112_column_icon"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="from_saltise",
            field=models.BooleanField(default=False),
        ),
    ]
